@@ -21,19 +21,27 @@ usage () {
     echo -e "\033[1mPARAMETERS:\033[0m"
     echo -e " \033[1m-u\033[0m               Update scripts: setup, quickinst, tz, km and helpers."
     echo -e ""
-    echo -e " \033[1m-latest\033[0m          Launch latest archboot environment (using kexec)."
-    echo -e "                  This operation needs at least \033[1m2.3 GB RAM\033[0m."
-    echo ""
-    echo -e " \033[1m-latest-install\033[0m  Launch latest archboot environment with downloaded"
-    echo -e "                  package cache (using kexec)."
-    echo -e "                  This operation needs at least \033[1m3.2 GB RAM\033[0m."
-    echo ""
-    echo -e " \033[1m-latest-image\033[0m    Generate latest image files in /archboot directory"
-    echo -e "                  This operation needs at least \033[1m3.9 GB RAM\033[0m."
-    echo ""
-    echo -e " \033[1m-launch-xfce\033[0m     Launch XFCE desktop with VNC sharing enabled."
-    echo -e "                  This operation needs at least \033[1m3.5 GB RAM\033[0m."
-    echo ""
+    if [[ "$(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g')" -gt 2000000 ]]; then
+        echo -e " \033[1m-latest\033[0m          Launch latest archboot environment (using kexec)."
+        echo -e "                  This operation needs at least \033[1m2.0 GB RAM\033[0m."
+        echo ""
+    fi
+    if [[ "$(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g')" -gt 3200000 ]]; then
+        echo -e " \033[1m-latest-install\033[0m  Launch latest archboot environment with downloaded"
+        echo -e "                  package cache (using kexec)."
+        echo -e "                  This operation needs at least \033[1m3.2 GB RAM\033[0m."
+        echo ""
+    fi
+    if [[ "$(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g')" -gt 3900000 ]]; then
+        echo -e " \033[1m-latest-image\033[0m    Generate latest image files in /archboot directory"
+        echo -e "                  This operation needs at least \033[1m3.9 GB RAM\033[0m."
+        echo ""
+    fi
+    if [[ "$(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g')" -gt 2500000 ]]; then
+        echo -e " \033[1m-launch-xfce\033[0m     Launch XFCE desktop with VNC sharing enabled."
+        echo -e "                  This operation needs at least \033[1m2.5 GB RAM\033[0m."
+        echo ""
+    fi
     echo -e " \033[1m-h\033[0m               This message."
     exit 0
 }
@@ -85,33 +93,62 @@ _update_installer_check() {
     fi
 }
 
+_zram_initialize() {
+    # add defaults
+    _ZRAM_ALGORITHM=${_ZRAM_ALGORITHM:-"zstd"}
+    if ! grep -qw zram /proc/modules; then
+        modprobe zram num_devices=2> /dev/tty7 2>&1
+        echo "${_ZRAM_ALGORITHM}" >/sys/block/zram0/comp_algorithm
+        echo "${_ZRAM_ALGORITHM}" >/sys/block/zram1/comp_algorithm
+    fi
+}
+
+# use -o discard for RAM cleaning on delete
+# (online fstrimming the block device!)
+# fstrim <mountpoint> for manual action
+# it needs some seconds to get RAM free on delete!
+_zram_usr() {
+    if ! mountpoint -q /usr; then
+        echo "${1}" >/sys/block/zram0/disksize
+        echo "Creating btrfs filesystem with ${1} on /dev/zram0 ..." > /dev/tty7
+        mkfs.btrfs -q --mixed /dev/zram0 > /dev/tty7 2>&1
+        mkdir /usr.zram
+        mount -o discard /dev/zram0 "/usr.zram" > /dev/tty7 2>&1
+        echo "Moving /usr to /usr.zram ..." > /dev/tty7
+        cp -r /usr/* /usr.zram/
+        ln -sfn /usr.zram/lib /lib
+        ln -sfn /usr.zram/lib /lib64
+        echo /usr.zram/lib > /etc/ld.so.conf
+        ldconfig
+        ln -sfn /usr.zram/bin /bin
+        ln -sfn /usr.zram/bin /sbin
+        #shellcheck disable=SC2115
+        rm -r /usr/*
+        /usr.zram/bin/./mount --bind /usr.zram /usr
+        systemctl daemon-reload > /dev/tty7 2>&1
+        systemctl restart dbus > /dev/tty7 2>&1
+    fi
+}
+
+_zram_w_dir() {
+    echo "${1}" >/sys/block/zram1/disksize
+    echo "Creating btrfs filesystem with ${1} on /dev/zram1 ..." > /dev/tty7
+    mkfs.btrfs -q --mixed /dev/zram1 > /dev/tty7 2>&1
+    [[ -d "${_W_DIR}" ]] || mkdir "${_W_DIR}"
+    mount -o discard /dev/zram1 "${_W_DIR}" > /dev/tty7 2>&1
+}
+
 _umount_w_dir() {
     if mountpoint -q "${_W_DIR}"; then
         echo "Unmounting ${_W_DIR} ..." > /dev/tty7
         # umount all possible mountpoints
         umount -R "${_W_DIR}"
-        echo 1 > /sys/block/zram0/reset
+        echo 1 > /sys/block/zram1/reset
         # wait 5 seconds to get RAM cleared and set free
         sleep 5
     fi
 }
 
-_zram_mount() {
-    # add defaults
-    _ZRAM_ALGORITHM=${_ZRAM_ALGORITHM:-"zstd"}
-    modprobe zram > /dev/tty7 2>&1
-    echo "${_ZRAM_ALGORITHM}" >/sys/block/zram0/comp_algorithm
-    echo "${1}" >/sys/block/zram0/disksize
-    echo "Creating btrfs filesystem with ${_DISKSIZE} on /dev/zram0 ..." > /dev/tty7
-    mkfs.btrfs -q --mixed /dev/zram0 > /dev/tty7 2>&1
-    [[ -d "${_W_DIR}" ]] || mkdir "${_W_DIR}"
-    # use -o discard for RAM cleaning on delete
-    # (online fstrimming the block device!)
-    # fstrim <mountpoint> for manual action
-    # it needs some seconds to get RAM free on delete!
-    mount -o discard /dev/zram0 "${_W_DIR}" > /dev/tty7 2>&1
-}
-
 _clean_archboot() {
     # remove everything not necessary
     rm -rf "/usr/lib/firmware"
@@ -196,37 +233,31 @@ _create_initramfs() {
     cd  "${_W_DIR}"/tmp || exit 1
     find . -mindepth 1 -printf '%P\0' | sort -z |
     bsdtar --uid 0 --gid 0 --null -cnf - -T - |
-    bsdtar --null -cf - --format=newc @- | zstd -T0 -10> /initrd.img &
-    sleep 2
-    for i in $(find . -mindepth 1 -type f | sort); do
-        rm "${i}" >/dev/null 2>&1
-        sleep 0.002
-    done
-    while pgrep -x bsdtar >/dev/null 2>&1; do
-        sleep 1
-    done
+    bsdtar --null -cf - --format=newc @- | zstd --rm -T0> /initrd.img
 }
 
-_kexec() {
-    # load kernel and initrds into running kernel in background mode!
-    kexec -l /"${VMLINUZ}" --initrd="/initrd.img" --reuse-cmdline&
-    # wait 2 seconds for getting a complete initramfs
-    # remove kernel and initrd to save RAM for kexec in background
-    sleep 2
-    rm /{initrd.img,${VMLINUZ}}
-    while pgrep -x kexec >/dev/null 2>&1; do
-        sleep 1
-    done
-    echo -e "\033[1mFinished:\033[0m Rebooting in a few seconds ..."
-    # don't show active prompt wait for kexec to be launched
-    while true; do
-        if [[ -e "/sys/firmware/efi" ]]; then
-            # UEFI kexec call
-            systemctl kexec 2>/dev/null
-        else
-            # BIOS kexec call
-            kexec -e 2>/dev/null
+_kexec () {
+    if [[ $(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g') -gt 4015000 ||\
+    $(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g') -lt 3200000 ]]; then
+        echo -e "\033[1mRunning kexec with new KEXEC_FILE_LOAD ...\033[0m"
+        # works on systems with >4GB, --latest-install needs around 4400MB RAM
+        # that causes a gap which can break qemu kexec or parallels desktop
+        if [[ $(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g') -lt 4400000 &&\
+        $(grep -w MemTotal /proc/meminfo | cut -d ':' -f2 | sed -e 's# ##g' -e 's#kB$##g') -gt 4015000 ]]; then
+            echo -e "\033[91mMemory Gap Warning (4.0G - 4.4G RAM):\033[0m"
+            echo -e "\033[1m- Possibility of not working kexec boot.\033[0m"
+            echo -e "\033[1m- Please use more or less RAM.\033[0m"
+            sleep 10
         fi
+        kexec -s -f /"${VMLINUZ}" --initrd="/initrd.img" --reuse-cmdline &
+    else
+        echo -e "\033[1mRunning kexec with old KEXEC_LOAD ...\033[0m"
+        # works on systems with <4GB
+        kexec -c -f /"${VMLINUZ}" --initrd="/initrd.img" --reuse-cmdline &
+        sleep 2
+        rm /{${VMLINUZ},initrd.img}
+    fi
+    while pgrep -x kexec > /dev/null 2>&1; do
         sleep 1
     done
 }
@@ -237,7 +268,17 @@ _cleanup_install() {
     rm -rf /usr/lib/libgo.*
 }
 
+_cleanup_cache() {
+    # remove packages from cache
+    #shellcheck disable=SC2013
+    for i in $(grep -w 'installed' /var/log/pacman.log | cut -d ' ' -f 4); do
+        rm -rf /var/cache/pacman/pkg/"${i}"-[0-9]*
+    done
+}
+
 _prepare_xfce() {
+    echo "Removing firmware files ..."
+    rm -rf /usr/lib/firmware
     # fix libs first, then install packages from defaults
     _XORG="${_FULL_PACKAGES} ${_X_PACKAGES}"
     # saving RAM by calling always cleanup hook and installing each package alone
@@ -251,6 +292,8 @@ _prepare_xfce() {
             #shellcheck disable=SC2086
             pacman -S ${i} --config ${_PACMAN_CONF} --noconfirm >/dev/null 2>&1 || exit 1
             _cleanup_install
+            _cleanup_cache
+            rm -f /var/log/pacman.log
         done
     else
         echo "Updating environment to latest packages (ignoring packages: ${_X_IGNORE}) ..."
@@ -268,14 +311,12 @@ _prepare_xfce() {
             #shellcheck disable=SC2086
             pacman -S ${i} --noconfirm >/dev/null 2>&1 || exit 1
             _cleanup_install
+            _cleanup_cache
+            rm -f /var/log/pacman.log
         done
     fi
-    echo "Removing files from cache directory: ${_X_RM_PACKAGES} ..."
-    for i in ${_X_RM_PACKAGES}; do
-        rm -f /var/cache/pacman/pkg/"${i}"-*
-    done
-    echo "Removing firmware files ..."
-    rm -rf /usr/lib/firmware
+    echo "Removing not used icons ..."
+    rm -rf /usr/share/icons/breeze-dark
     echo "Recreating C.UTF-8 locale ..."
     sed -i -e 's:#C.UTF-8 UTF-8:C.UTF-8 UTF-8:g' "${1}/etc/locale.gen"
     locale-gen >/dev/null 2>&1
@@ -510,7 +551,7 @@ EOF
     sed -i -e 's#Categories=.*#Categories=X-Xfce-Toplevel;#g' /usr/share/applications/gparted.desktop
     echo "Hiding ${_HIDE_MENU} menu entries ..."
     for i in ${_HIDE_MENU}; do
-        echo 'NoDisplay=true' >> /usr/share/applications/$i.desktop
+        echo 'NoDisplay=true' >> /usr/share/applications/"${i}".desktop
     done
     echo "Autostarting setup ..."
     cat << EOF > /etc/xdg/autostart/archboot.desktop
@@ -529,6 +570,4 @@ Type=Application
 Name=Tigervnc
 Exec=x0vncserver -rfbauth /etc/tigervnc/passwd
 EOF
-    echo "Setting VNC password /etc/tigervnc/passwd to ${_VNC_PW} ..."
-    echo "${_VNC_PW}" | vncpasswd -f > /etc/tigervnc/passwd
 }
